(* File lexer.mll *)
(* (C) 20080526- Kim, Taegyoon *)
{
  open Parser        (* The type token is defined in parser.mli *)
}
rule token = parse
    "display" { DISPLAY }
  | "newline" { NEWLINE }
  | "car" { CAR }
  | "cdr" { CDR }
  | [' ' '\t' '\n' '\r']     { token lexbuf }     (* skip blanks *)
  | ['0'-'9']+ as lxm { INTNUM (int_of_string lxm) }
  | ['A'-'Z' 'a'-'z']+ as lxm { ID lxm } (* identifier *)
  | '+' { PLUS }
  | '-' { MINUS }
  | '*' { TIMES }
  | '/' { DIV }
  | "&&" { ANDAND }
  | "||" { OROR }
  | "!" { NOT }
  | '=' { EQ }
  | '<' { LT }
  | '>' { GT }
  | "<=" { LTE }
  | ">=" { GTE }
  | "!=" { NEQ }
  | "==" { EQEQ }
  | ';' { SEMICOLON }
  | '(' { LPAREN }
  | ')' { RPAREN }
  | '{' { LCURLY }
  | '}' { RCURLY }
  | "#" { SHARP }
  | '"' ([^'"']* as lxm) '"' { STRING lxm } (* string literal *)
  | "?" { QUESTION }
  | "\\" { LAMBDA }
  | eof { EOF }
