/* File parser.mly */
/* (C) 20080402 Kim, Taegyoon */
%{
  (* for debugging *)
  let ps s = print_string s; flush stdout

  (* makes a closure *)
  let delay : string list -> string list = fun
    s -> ["(lazy ("] @ s @ ["))"]
(*    s -> ["(fun () -> "] @ s @ [")"] *)

  let paren : string list -> string list = fun
    s -> ["("] @ s @ [")"]

  (* false->0, true->1 *)
  let int_of_bool b = if b then 1 else 0
%}

%token EOF
%token PLUS MINUS TIMES DIV
%token LPAREN RPAREN LCURLY RCURLY
%token SEMICOLON
%token ANDAND OROR NOT
%token LT GT LTE GTE EQEQ NEQ
%token EQ
%token DISPLAY
%token INT
%token RAND
%token <int> INTNUM
%token <string> ID
%token SHARP
%token <string> STRING
%token CAR CDR
%token NEWLINE
%token QUESTION
%token LAMBDA

%start main             /* the entry point */
%type <string list> main

%%

main:
expr { 
  [ "(* This file is generated by tg *)\n";
    "open Evaluator\n"
  ] @
  ["let _ = try\n"] @ $1 @ ["\n"] @
    ["with TypeError (a,b) -> print_string (typeErrorInfo a b); print_newline ()\n"]
}
;

id:
  ID { [$1] }
;

expr:
  INTNUM { delay (["Int "] @ [string_of_int $1]) }
  | STRING { delay (["String \""] @ [$1] @ ["\""]) }
  | id { $1 }
  | expr expr CAR { paren (["car "] @ $1 @ [" "] @ $2) }
  | expr expr CDR { paren (["cdr "] @ $1 @ [" "] @ $2) }
  | expr DISPLAY { ["(display "] @ $1 @ [")"] }
  | NEWLINE { ["(newline ())"] }
  | expr expr PLUS { paren (["add "] @ $1 @ [" "] @ $2) }
  | expr expr MINUS { paren (["sub "] @ $1 @ [" "] @ $2) }
  | expr expr TIMES { paren (["mul "] @ $1 @ [" "] @ $2) }
  | expr expr DIV { paren (["div "] @ $1 @ [" "] @ $2) }
  | expr expr EQ { paren (["is_equal "] @ $1 @ [" "] @ $2) }
  | expr expr expr QUESTION { paren (["if_then_else "] @ $1 @ [" "] @ $2 @ [" "] @ $3) }
/*  | func { $1 } */
  | expr func { paren ($2 @ [" "] @ $1) }
  | expr expr SEMICOLON { $1 @ [";\n"] @ $2 }     
;

func:
  expr id LAMBDA { paren (["fun "] @ $2 @ [" -> "] @ $1) }
;
